@@ -1,32 +1,28 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-"""Python 3 LSV2 library
-   This library is an attempt to implement the LSV2 communication protocol used by certain
-   CNC controls. It's goal is to transfer file between the application and the control as well
-   as collect information about said files.
-   Most of this library is based on the work of tfischer73 and his Eclipse
-   plugin https://github.com/tfischer73/Eclipse-Plugin-Heidenhain . Since I could not find any
-   documentation beside the plugin some parts are based on re-engineering and might therefore be
-   not correct.
-   Everything related to unknown/untested System functions was left out as these function
-   might compromise the control.
+"""This library is an attempt to implement the LSV2 communication protocol used by certain
+   CNC controls.
+   Please consider the dangers of using this library on a production machine! This library is
+   by no means complete and could damage the control or cause injuries! Everything beyond simple
+   file manipulation is blocked by a lockout parameter. Use at your own risk!
 """
 import logging
+import re
 import struct
 import warnings
 from pathlib import Path
 
 from . import const as L_C
 from .low_level_com import LLLSV2Com
-from .misc import (decode_directory_info, decode_file_system_info,
-                   decode_system_parameters, decode_tool_information, decode_override_information)
+from .misc import (decode_directory_info, decode_error_message,
+                   decode_file_system_info, decode_override_information,
+                   decode_system_parameters, decode_tool_information)
 from .translate_messages import (get_error_text, get_execution_status_text,
                                  get_program_status_text)
 
 
 class LSV2():
-    """Implementation of the LSV2 protocol used to communicate with certain CNC controls.
-       This is just a test implementation that will get worked into a complete Python library."""
+    """Implementation of the LSV2 protocol used to communicate with certain CNC controls"""
 
     DRIVE_TNC = 'TNC:'
     DRIVE_TNC = 'PLC:'
@@ -261,7 +257,7 @@ class LSV2():
             logging.info(
                 'safe mode is off, login and system commands are not restricted. Use with caution!')
             self._known_logins = (L_C.LOGIN_INSPECT, L_C.LOGIN_DIAG, L_C.LOGIN_PLCDEBUG, L_C.LOGIN_FILETRANSFER, L_C.LOGIN_MONITOR, L_C.LOGIN_DSP,
-                                  L_C.LOGIN_DNC, L_C.LOGIN_SCOPE, L_C.LOGIN_STREAMAXES, L_C.LOGIN_FILEPLC, L_C.LOGIN_FILESYS)
+                                  L_C.LOGIN_DNC, L_C.LOGIN_SCOPE, L_C.LOGIN_STREAMAXES, L_C.LOGIN_FILEPLC, L_C.LOGIN_FILESYS, L_C.LOGIN_FILELOG)
             self._known_sys_cmd = (LSV2.SYSCMD_RESET_TNC, LSV2.SYSCMD_STOP_TIMEUPDATE, LSV2.SYSCMD_SET_BUF1024, LSV2.SYSCMD_SET_BUF512,
                                    LSV2.SYSCMD_SET_BUF2048, LSV2.SYSCMD_SET_BUF3072, LSV2.SYSCMD_SET_BUF4096, LSV2.SYSCMD_SECURE_FILE_SEND,
                                    LSV2.SYSCMD_RESET_DNC, LSV2.SYSCMD_RESET_LSV2, LSV2.SYSCMD_UPDATE_TNCOPT, LSV2.SYSCMD_PUSH_PRESET_INTO_LOG,
@@ -272,6 +268,7 @@ class LSV2():
         self._sys_par = None
         self._secure_file_send = False
         self._control_type = L_C.TYPE_UNKNOWN
+        self._last_error_code = None
 
     def connect(self):
         """connect to control"""
@@ -320,18 +317,20 @@ class LSV2():
 
             if response in expected_response:
                 if content is not None and len(content) > 0:
-                    logging.info(
+                    logging.debug(
                         'command %s executed successfully, received %s with %d bytes payload', command, response, len(content))
                     return content
-                logging.info(
+                logging.debug(
                     'command %s executed successfully, received %s without any payload', command, response)
                 return True
 
             if response in LSV2.RESPONSE_T_ER:
                 self._decode_error(content)
+                self._last_error_code = struct.unpack('!BB', content)
             else:
                 logging.error(
                     'recived unexpected response %s to command %s. response code %s', response, command, content)
+                self._last_error_code = None
 
         return False
 
@@ -344,6 +343,8 @@ class LSV2():
 
         if response in LSV2.RESPONSE_T_ER:
             self._decode_error(content)
+        elif response in self.RESPONSE_T_FD:
+            logging.debug('Transfer is finished with no content')
         elif response not in expected_response:
             logging.error(
                 'recived unexpected response %s block read for command %s. response code %s', response, command, content)
@@ -741,7 +742,7 @@ class LSV2():
         result = self._send_recive(
             LSV2.COMMAND_R_FI, LSV2.RESPONSE_S_FI, payload=payload)
         if result:
-            file_info = decode_file_system_info(result)
+            file_info = decode_file_system_info(result, self._control_type)
             logging.debug(
                 'successfuly received file information %s', file_info)
             return file_info
@@ -751,7 +752,9 @@ class LSV2():
         return False
 
     def get_directory_content(self):
-        """Query content of current working directory from the control
+        """Query content of current working directory from the control.
+        In some situations it is necessary to fist call get_directory_info() or else
+        the attributes won't be correct.
 
         :returns: list of dict with info about directory entries
         :rtype: list
@@ -765,7 +768,8 @@ class LSV2():
         logging.debug(
             'received %d entries for directory content information', len(result))
         for entry in result:
-            dir_content.append(decode_file_system_info(entry))
+            dir_content.append(decode_file_system_info(
+                entry, self._control_type))
 
         logging.debug(
             'successfuly received directory information %s', dir_content)
@@ -1075,9 +1079,7 @@ class LSV2():
             LSV2.COMMAND_R_FL, payload, buffer_size=self._buffer_size)
 
         with local_file.open('wb') as out_file:
-            #file_buffer = bytearray()
             if response in self.RESPONSE_S_FL:
-                # file_buffer.extend(content)
                 if binary_mode:
                     out_file.write(content)
                 else:
@@ -1089,7 +1091,6 @@ class LSV2():
                     response, content = self._llcom.telegram(
                         LSV2.RESPONSE_T_OK, payload=None, buffer_size=self._buffer_size)
                     if response in self.RESPONSE_S_FL:
-                        # file_buffer.extend(content)
                         if binary_mode:
                             out_file.write(content)
                         else:
@@ -1111,9 +1112,11 @@ class LSV2():
                 if response in self.RESPONSE_T_ER or response in self.RESPONSE_T_BD:
                     logging.error('an error occurred while loading the first block of data for file %s : %s',
                                   remote_path, self._decode_error(content))
+                    self._last_error_code = struct.unpack('!BB', content)
                 else:
                     logging.error(
                         'could not load file with error %s', response)
+                    self._last_error_code = None
                 return False
 
         logging.info('received %d bytes transfer complete for file %s to %s',
@@ -1395,3 +1398,91 @@ class LSV2():
         logging.warning(
             'an error occurred while querying current override information. This does not work for all control types')
         return False
+
+    def get_error_messages(self):
+        """Get information about the first or next error displayed on the control
+
+        :param bool next_error: if True check if any further error messages are available
+        :returns: error information or False if something went wrong
+        :rtype: dict
+        """
+        messages = list()
+        self.login(login=L_C.LOGIN_DNC)
+
+        payload = bytearray()
+        payload.extend(struct.pack('!H', L_C.RUN_INFO_FIRST_ERROR))
+        result = self._send_recive(
+            LSV2.COMMAND_R_RI, LSV2.RESPONSE_S_RI, payload)
+        if result:
+            messages.append(decode_error_message(result))
+            payload = bytearray()
+            payload.extend(struct.pack('!H', L_C.RUN_INFO_NEXT_ERROR))
+            result = self._send_recive(
+                LSV2.COMMAND_R_RI, LSV2.RESPONSE_S_RI, payload)
+            logging.debug('successfuly read first error but further errors')
+
+            while result:
+                messages.append(decode_error_message(result))
+                result = self._send_recive(
+                    LSV2.COMMAND_R_RI, LSV2.RESPONSE_S_RI, payload)
+
+            if self._last_error_code[1] == L_C.LSV2_ERROR_T_ER_NO_NEXT_ERROR:
+                logging.debug('successfuly read all errors')
+            else:
+                logging.warning(
+                    'an error occurred while querying error information.')
+
+            return messages
+
+        elif self._last_error_code[1] == L_C.LSV2_ERROR_T_ER_NO_NEXT_ERROR:
+            logging.debug('successfuly read first error but no error active')
+            return messages
+
+        logging.warning(
+            'an error occurred while querying error information. This does not work for all control types')
+
+        return False
+
+    def _walk_dir(self, descend=True):
+        """helber function to recursively search in directories for files
+
+        :param bool descend: control if search should run recursively
+        :returns: list of files found in directory
+        :rtype: list
+        """
+        current_path = self.get_directory_info()['Path']
+        content = list()
+        for entry in self.get_directory_content():
+            if entry['Name'] == '.' or entry['Name'] == '..' or entry['Name'].endswith(':'):
+                continue
+            if entry['is_directory'] is True and descend is True:
+                if self.change_directory(current_path + entry['Name']):
+                    content.extend(self._walk_dir())
+            else:
+                content.append(current_path + entry['Name'])
+        self.change_directory(current_path)
+        return content
+
+    def get_file_list(self, path=None, descend=True, pattern=None):
+        """Get list of files in directory structure.
+
+        :param str path: path of the directory where files should be searched. if None than the current directory is used
+        :param bool descend: control if search should run recursiv
+        :param str pattern: regex string to filter the file names
+        :returns: list of files found in directory
+        :rtype: list
+        """
+        if path is not None:
+            if self.change_directory(path) is False:
+                logging.warning('could not change to directory')
+                return None
+
+        if pattern is None:
+            file_list = self._walk_dir(descend)
+        else:
+            file_list = list()
+            for entry in self._walk_dir(descend):
+                file_name = entry.split('/')[-1]
+                if re.match(pattern, file_name):
+                    file_list.append(entry)
+        return file_list
