@@ -22,7 +22,7 @@ class TableReader():
 
         :param str or Path table_path: Path to the table file
 
-        :returns: list od dictionarys. key is the column name, value the content of the table cell
+        :returns: list od dictionaries. key is the column name, value the content of the table cell
         :rtype: list
         """
         self.name = None
@@ -54,15 +54,15 @@ class TableReader():
                 if 'MM' in header.group('unit'):
                     self.is_metric = True
                     logging.debug(
-                        'Header Infomation for file "%s" Name "%s", file is metric, Version: "%s"', table_file, self.name, self.version)
+                        'Header Information for file "%s" Name "%s", file is metric, Version: "%s"', table_file, self.name, self.version)
                 else:
                     self.is_metric = False
                     logging.debug(
-                        'Header Infomation for file "%s" Name "%s", file is inch, Version: "%s"', table_file, self.name, self.version)
+                        'Header Information for file "%s" Name "%s", file is inch, Version: "%s"', table_file, self.name, self.version)
             else:
                 self.has_unit = False
                 self.is_metric = False
-                logging.debug('Header Infomation for file "%s" Name "%s", file has no units, Version: "%s"',
+                logging.debug('Header Information for file "%s" Name "%s", file has no units, Version: "%s"',
                               table_file, self.name, self.version)
 
             next_line = tfp.readline()
@@ -84,7 +84,7 @@ class TableReader():
             for column_match in column_pattern.finditer(column_header):
                 column_list.append({'start': column_match.start(
                 ), 'end': column_match.end()-1, 'name': column_match.group().strip()})
-                
+
             logging.debug('Found %d columns', len(column_list))
 
             table_content = list()
@@ -105,7 +105,7 @@ class TableReader():
 
     @staticmethod
     def format_entry_float(str_value):
-        """convert the string value o a table cell to float value"""
+        """convert the string value of a table cell to float value"""
         str_value = str_value.strip()
         if str_value == '-' or len(str_value) == 0:
             return None
@@ -114,7 +114,7 @@ class TableReader():
 
     @staticmethod
     def format_entry_int(str_value):
-        """convert the string value o a table cell to int value"""
+        """convert the string value of a table cell to int value"""
         str_value = str_value.strip()
         if str_value == '-' or len(str_value) == 0:
             return None
@@ -123,7 +123,7 @@ class TableReader():
 
     @staticmethod
     def format_entry_bool(str_value):
-        """convert the string value o a table cell to boolean value"""
+        """convert the string value of a table cell to boolean value"""
         str_value = str_value.strip()
         if len(str_value) == 0:
             return None
