@@ -1,6 +1,8 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-
+"""This script contains examples on how to use different functions of pyLSV2
+   Not all functions are shown here, especially the file functions aren't shown here
+"""
 import logging
 import sys
 import tempfile
@@ -9,7 +11,7 @@ from pathlib import Path
 
 import pyLSV2
 
-logging.basicConfig(level=logging.DEBUG)
+logging.basicConfig(level=logging.INFO)
 
 if __name__ == "__main__":
     address = '192.168.56.101'
@@ -41,12 +43,12 @@ if __name__ == "__main__":
     else:
         # old style
         print('Current Language: {}'.format(con.get_machine_parameter('7230.0')))
-    
+
     # changing the value of a machine parameter
-    #con.login(pyLSV2.LOGIN_PLCDEBUG)
-    #new style: con.set_machine_parameter('CfgDisplayLanguage.ncLanguage', 'CZECH', safe_to_disk=False)
-    #old style: con.set_machine_parameter('7230.0', '2', safe_to_disk=False)
-    #con.logout(pyLSV2.LOGIN_PLCDEBUG)
+    # con.login(pyLSV2.LOGIN_PLCDEBUG)
+    # new style: con.set_machine_parameter('CfgDisplayLanguage.ncLanguage', 'CZECH', safe_to_disk=False)
+    # old style: con.set_machine_parameter('7230.0', '2', safe_to_disk=False)
+    # con.logout(pyLSV2.LOGIN_PLCDEBUG)
 
     # read program stack
     print('Current Program Stack: {}'.format(con.get_program_stack()))
@@ -81,10 +83,25 @@ if __name__ == "__main__":
 
         with tempfile.TemporaryDirectory(suffix=None, prefix='pyLSV2_') as tmp_dir_name:
             local_recive_path = Path(tmp_dir_name).joinpath('tool_p.tch')
-            con.recive_file(local_path=str(local_recive_path), remote_path=pocket_table_path, binary_mode=transfer_binary)
+            con.recive_file(local_path=str(local_recive_path),
+                            remote_path=pocket_table_path,
+                            binary_mode=transfer_binary)
             tr = pyLSV2.TableReader()
             pockets = tr.parse_table(local_recive_path)
             spindel = list(filter(lambda pocket: pocket['P'] == spindel_lable, pockets))[0]
             print('Current tool number {T} with name {TNAME}'.format(**spindel))
 
+    # read error messages via LSV2, works only on iTNC controls
+    if con.is_itnc():
+        e_m = con.get_error_messages()
+        print('Number of currently ative error messages: {:d}'.format(len(e_m)))
+        for i, msg in enumerate(e_m):
+            print('Error {:d} : {:s}'.format(i, msg['Text']))
+
+    # list all NC-Programms in TNC partition
+    h_files = con.get_file_list(path='TNC:', pattern=r'[\$A-Za-z0-9_-]*\.[hH]$')
+    print('Found {:d} Klartext programms: {:}'.format(len(h_files), h_files))
+    i_files = con.get_file_list(path='TNC:', pattern=r'[\$A-Za-z0-9_-]*\.[iI]$')
+    print('Found {:d} DIN/ISO programms: {:}'.format(len(i_files), i_files))
+
     con.disconnect()
